
/*
   wmch : A Window Manager CHooser
   Copyright (C) 1997  Dominique Rousseau <rousseau@neuronnexion.fr>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Parts from GUBI are Copyright (C) Tim Janik  <timj@psynet.net>
 */

#include <stdio.h>
#include <ctype.h>
#include <glib.h>

#include "rcparser.h"

void wmch_parse_line(char *string, char **v, int wanted, int *c);

void wmch_parse_line(char *string, char **v, int wanted, int *c)
{
	char *ptr;
	char waited;

	*c = 0;

	g_return_if_fail(string != NULL);

	ptr = string;

	/* Search tokens */
	while (*c < wanted) {
		/* Skip leading spaces */
		while (isspace(*ptr)) {
			if (*ptr == '\0')
				return;
			ptr++;
		}

		/* Check length */
		g_return_if_fail(strlen(ptr) > 0);

		/* Skip if comment line */
		if (*ptr == '#')
			return;

		/* Find which chracter will finish the current token */
		switch (*ptr) {
		case '"':
		case '\'':
			waited = *ptr;
			ptr++;
			break;

		default:
			waited = ' ';
			break;
		}

		/* Set token in the vbector */
		v[*c] = ptr;
		/* Count 1 */
		(*c)++;

		if ((waited == ' ') && (*c == wanted))
			return;

		/* Search end of token */
		while ((waited == ' ') ? !isspace(*ptr) : (*ptr != waited)) {
			if (*ptr == '\0')
				return;

			/* Go on */
			ptr++;
		}
		/* Put end of token */
		*ptr = '\0';
		ptr++;
	}

	return;
}

void wmch_rcparse(char *filename, rc_struct ** lines, int *count)
{
	FILE *fic;
	char line[256], **decoup;
	char *new_str;
	int n;

	/* Init */
	*count = 0;
	*lines = NULL;
	decoup = g_new(char *, 4);

	/* Open file */
	fic = fopen(filename, "rt");
	g_return_if_fail(fic != NULL);

	/* Read it */
	while (!feof(fic)) {
		/* Get line */
		if (fgets(line, 255, fic) != NULL) {
			/* And parse it */
			wmch_parse_line(line, decoup, 3, &n);
			if (n == 3) {
				/* Get memory to hold info */
				*lines = g_realloc(*lines, sizeof(rc_struct) * (*count + 1));

				/* The Label */
				new_str = g_new(char, strlen(decoup[0]) + 1);
				strcpy(new_str, decoup[0]);
				(*lines)[*count].label = new_str;

				/* The Pixmap */
				new_str = g_new(char, strlen(decoup[1]) + 1);
				strcpy(new_str, decoup[1]);
				(*lines)[*count].pixmap = new_str;

				/* The Command */
				new_str = g_new(char, strlen(decoup[2]) + 1);
				strcpy(new_str, decoup[2]);
				(*lines)[*count].command = new_str;

				(*count)++;
			}
		}
	}

	fclose(fic);

	g_free(decoup);
}
